@@ -42,7 +42,6 @@ char *banner="% Written by DMP, Version 0.64";	/* first line of output */
 char *term_banner="This is DMP, Version 0.64";
 
 #include "config.h"
-#include <kpathsea/c-auto.h>
 #include <kpathsea/c-proto.h>
 #include <kpathsea/tex-file.h>
 #include <kpathsea/c-ctype.h>
@@ -1204,7 +1203,7 @@ void dmp_usage(name, status)
     char *name;
     int status;
 {
-    extern DllImport char *kpse_bug_address;
+    extern KPSEDLL char *kpse_bug_address;
     FILE *f = status == 0 ? stdout : stderr;
     fputs ("Usage: dmp [OPTION]... DITROFFFILE [MPXFILE]\n\
 !   Translate DITROFFFILE to the MetaPost MPXFILE or standard output.\n\
@@ -1216,7 +1215,6 @@ void dmp_usage(name, status)
     exit(status);
 }
 
-
 int main(argc, argv)
     int argc;
     char *argv[];
