@@ -1,51 +1,80 @@
 #include "libpdftex.h"
-#include "image.h"
 #include <kpathsea/c-vararg.h>
 #include <kpathsea/c-proto.h>
 
 char *filename = 0;
-char printf_buf[PRINTF_BUF_SIZE];
+char print_buf[1024];
 
-void __cdecl writeEPDF(char *fmt, ...) {
-    va_list args;
-
-    va_start(args, fmt);
-    vsprintf(printf_buf, fmt, args);
-    flush_printf_buf();                                    
-    va_end(args);
-}
-
-void __cdecl fail(char *fmt, ...) {
-    va_list args;
-
-    fprintf(stderr, "\nError: %s", program_invocation_name);
-    if (filename)
-        fprintf(stderr, " (file %s)", filename);
-    fprintf(stderr, ":\n");
-
-    va_start(args, fmt);
-    vfprintf(stderr, fmt, args);
-    va_end(args);
-
-    fprintf(stderr, "\n");
-    exit(-1);
-}
 
 integer pdfoffset()
 {
     return pdfgone + pdfptr;
 }
 
-void pdfout(int c) {
+void pdfout(integer c)
+{
     pdfbuf[pdfptr++] = c;
     if (pdfptr == pdfbufsize)
         pdfflush();
 }
 
-void flush_printf_buf()
+void pdf_printf(char *fmt,...)
 {
-    char *p = printf_buf;
-    if (sizeof(pdfbuf) - pdfptr <= PRINTF_BUF_SIZE)
+    va_list args;
+    va_start(args, fmt);
+    vsprintf(print_buf, fmt, args);
+    flush_print_buf();                                    
+    va_end(args);
+}
+
+void tex_printf(char *fmt,...)
+{
+    va_list args;
+    va_start(args, fmt);
+    vsprintf(print_buf, fmt, args);
+    print(maketexstring(print_buf));
+    flushstring();
+    xfflush(stdout);
+    va_end(args);
+}
+
+void pdftex_fail(char *fmt,...)
+{
+    va_list args;
+    va_start(args, fmt);
+    println();
+    tex_printf("Error: %s", program_invocation_name);
+    if (filename)
+        tex_printf(" (file %s)", filename);
+    tex_printf(": ");
+    vsprintf(print_buf, fmt, args);
+    print(maketexstring(print_buf));
+    flushstring();
+    va_end(args);
+    println();
+    exit(-1);
+}
+
+void pdftex_warn(char *fmt,...)
+{
+    va_list args;
+    va_start(args, fmt);
+    println();
+    tex_printf("Warning: %s", program_invocation_name);
+    if (filename)
+        tex_printf(" (file %s)", filename);
+    tex_printf(": ");
+    vsprintf(print_buf, fmt, args);
+    print(maketexstring(print_buf));
+    flushstring();
+    va_end(args);
+    println();
+}
+
+void flush_print_buf()
+{
+    char *p = print_buf;
+    if (sizeof(pdfbuf) - pdfptr <= 1024)
         pdfflush();
     while (*p)
         pdfbuf[pdfptr++] = *p++;
@@ -63,14 +92,12 @@ strnumber maketexstring(char *s)
     return makestring();
 }
 
-#define C_STR_BUF_SIZE  1024
-
 char *makecstring(integer s)
 {
-    static char cstrbuf[C_STR_BUF_SIZE];
+    static char cstrbuf[1024];
     char *p = cstrbuf;
     int i, l = strstart[s + 1] - strstart[s];
-    CHECK_BUF(l, C_STR_BUF_SIZE);
+    CHECK_BUF(l, 1024);
     for (i = 0; i < l; i++)
         *p++ = strpool[i + strstart[s]];
     *p = 0;
@@ -91,14 +118,14 @@ boolean str_eq_cstr(strnumber n, char *s)
 size_t xfwrite(void *ptr, size_t size, size_t nmemb, FILE *stream)
 {
     if (fwrite(ptr, size, nmemb, stream) != nmemb)
-        FAIL("fwrite() failed");
+        pdftex_fail("fwrite() failed");
     return nmemb;
 }
 
 int xfflush(FILE *stream)
 {
     if (fflush(stream) != 0)
-        FAIL("fflush() failed");
+        pdftex_fail("fflush() failed");
     return 0;
 }
 
@@ -106,7 +133,7 @@ int xgetc(FILE *stream)
 {
     int c = getc(stream);
     if (c < 0 && c != EOF)
-        FAIL("getc() failed");
+        pdftex_fail("getc() failed");
     return c;
 }
 
@@ -114,7 +141,7 @@ int xputc(int c, FILE *stream)
 {
     int i = putc(c, stream);
     if (i < 0)
-        FAIL("putc() failed");
+        pdftex_fail("putc() failed");
     return i;
 }
 
