@@ -16,19 +16,15 @@ key_entry font_keys[FONT_KEYS_NUM] = {
 
 void print_key(integer code, integer v)
 {
-    PDF_PRINTF("/%s " AND font_keys[code].pdfname);
+    pdf_printf("/%s ", font_keys[code].pdfname);
     if (!font_keys[code].valid) {
-      /*
-       * WARN("cannot read key `%s' from font file\nused value from %s.tfm"
-       *      AND font_keys[code].pdfname AND makecstring(fontname[tex_font]));
-       */
-        PDF_PRINTF("%li" AND
+        pdf_printf("%li",
                    (code == ITALIC_ANGLE_CODE) ? (long int)v :
                    (long int)dividescaled(v, pdffontsize[tex_font], 3));
     }
     else 
-        PDF_PRINTF("%li" AND (long int)font_keys[code].value.i);
-    PDF_PRINTF("\n");
+        pdf_printf("%li", (long int)font_keys[code].value.i);
+    pdf_printf("\n");
 }
 
 integer getitalicangle(internalfontnumber f)
@@ -41,7 +37,7 @@ integer getstemv(internalfontnumber f)
     return getquad(f)/10;
 }
 
-void getbbox(int f)
+void getbbox(internalfontnumber f)
 {
     font_keys[FONTBBOX1_CODE].value.i = 0;
     font_keys[FONTBBOX2_CODE].value.i = 
@@ -69,15 +65,26 @@ static char *subset_prefix(internalfontnumber f)
     return xstrdup(buf);
 }
 
-void dopdffont(int objnum, internalfontnumber f)
+void dopdffont(integer objnum, internalfontnumber f)
 {
     int k, e = 0;
+    char *p, *r, buf[1024];
     tex_font = f;
     filename = 0;
-    for (k = fontbc[f]; !pdfischarused(f, k); k++);
-    fontbc[f] = k;
-    for (k = fontec[f]; !pdfischarused(f, k); k--);
-    fontec[f] = k;
+    if (pdffontmap[f] == -1) {
+        if (pdfexpandfont[f] == 0)
+            pdftex_fail("pdffontmap[%i] not initialized", (int)f);
+        else if (pdfexpandfont[f] != 0) {
+            strcpy(buf, makecstring(fontname[f]));
+            p = strend(buf);
+            for (r = p - 1; r > buf && isdigit(*r); r--);
+            if (r == buf || r == p - 1 || (*r != '+' && *r != '-'))
+                pdftex_fail("invalid name of expanded font (%s)", buf);
+            *r = 0;
+            pdffontmap[f] = fmlookup(maketexstring(buf));
+            flushstring();
+        }
+    }
     if (pdffontmap[f] >= 0)
         fm_cur = fm_tab + pdffontmap[f];
     else
@@ -90,18 +97,23 @@ void dopdffont(int objnum, internalfontnumber f)
         fm_cur->prefix = subset_prefix(f);
     if (is_reencoded())
         e = enc_objnum(fm_cur->encoding);
+    if (pdfexpandfont[f] != 0 && is_included())
+        fm_cur = fm_ext_entry(f);
     pdfbegindict(objnum);
-    PDF_PRINTF("/Type /Font\n/Subtype /%s\n" AND
+    pdf_printf("/Type /Font\n");
+    if (pdfexpandfont[f] != 0)
+        pdf_printf("%% expand ratio = %i\n", (int)pdfexpandfont[f]);
+    pdf_printf("/Subtype /%s\n",
                is_truetype() ? "TrueType" : "Type1");
     if (is_reencoded() && !is_truetype())
-        PDF_PRINTF("/Encoding %li 0 R\n" AND (long int)e);
+        pdf_printf("/Encoding %li 0 R\n", (long int)e);
     if (is_basefont()) {
-        PDF_PRINTF("/BaseFont /%s\n>> endobj\n" AND fm_cur->base_name);
+        pdf_printf("/BaseFont /%s\n>> endobj\n", fm_cur->base_name);
         return;
     }
-    PDF_PRINTF("/FirstChar %li\n/LastChar %li\n/Widths %li 0 R\n" AND
-               (long int)fontbc[tex_font] AND (long int)fontec[tex_font]
-               AND (long int)(objptr + 1));
+    pdf_printf("/FirstChar %li\n/LastChar %li\n/Widths %li 0 R\n",
+               (long int)fontbc[tex_font], (long int)fontec[tex_font]
+, (long int)(objptr + 1));
 /*
  *    if font is included then
  *        |obj_ptr + 1| is the width array,
@@ -122,46 +134,39 @@ void dopdffont(int objnum, internalfontnumber f)
  *             |obj_ptr + 3| is the font descriptor,
  */
  
-    if (is_included())
+    if (is_included()) {
         if (is_truetype())
-            PDF_PRINTF("/BaseFont %li 0 R\n/FontDescriptor %li 0 R\n" AND
-                       (long int)(objptr + 4) AND (long int)(objptr + 5));
+            pdf_printf("/BaseFont %li 0 R\n/FontDescriptor %li 0 R\n",
+                       (long int)(objptr + 4), (long int)(objptr + 5));
         else
-            PDF_PRINTF("/BaseFont %li 0 R\n/FontDescriptor %li 0 R\n" AND
-                       (long int)(objptr + 7) AND (long int)(objptr + 8));
+            pdf_printf("/BaseFont %li 0 R\n/FontDescriptor %li 0 R\n",
+                       (long int)(objptr + 7), (long int)(objptr + 8));
+    }
     else if (!is_noparsing())
-        PDF_PRINTF("/BaseFont %li 0 R\n/FontDescriptor %li 0 R\n" AND
-                   (long int)(objptr + 2) AND (long int)(objptr + 3));
+        pdf_printf("/BaseFont %li 0 R\n/FontDescriptor %li 0 R\n",
+                   (long int)(objptr + 2), (long int)(objptr + 3));
 	else
-        PDF_PRINTF("/BaseFont /%s\n" AND fm_cur->base_name);
-    PDF_PRINTF(">> endobj\n");
+        pdf_printf("/BaseFont /%s\n", fm_cur->base_name);
+    pdf_printf(">> endobj\n");
     pdfnewobj(0, 0); /* chars width array */
-    PDF_PRINTF("[ ");
-    if (pdfextendfont[tex_font] == 1000) {
-        for (k = fontbc[tex_font]; k <= fontec[tex_font]; k++)
-            PDF_PRINTF("%li " AND
-                       (long int)dividescaled(getcharwidth(tex_font, k),
-                                              pdffontsize[tex_font], 3));
-    }
-    else {
-        for (k = fontbc[tex_font]; k <= fontec[tex_font]; k++)
-            PDF_PRINTF("%li " AND
-                       (long int)dividescaled(getcharwidth(tex_font, k),
-            xnoverd(pdffontsize[tex_font], pdfextendfont[tex_font], 1000), 3));
-    }
-    PDF_PRINTF("]\nendobj\n");
+    pdf_printf("[ ");
+    for (k = fontbc[tex_font]; k <= fontec[tex_font]; k++)
+        pdf_printf("%li ",
+                    (long int)dividescaled(getcharwidth(tex_font, k),
+                                           pdffontsize[tex_font], 3));
+    pdf_printf("]\nendobj\n");
     if (is_noparsing())
         return;
     if (is_included()) {
         pdfnewdict(0, 0); /* font file stream */
         if (!is_truetype())
-            PDF_PRINTF("/Length %li 0 R\n/Length1 %li 0 R\n/Length2 %li 0 R\n/Length3 %li 0 R\n" AND
-                       (long int)(objptr + 1) AND (long int)(objptr + 2) AND
-                       (long int)(objptr + 3) AND (long int)(objptr + 4));
+            pdf_printf("/Length %li 0 R\n/Length1 %li 0 R\n/Length2 %li 0 R\n/Length3 %li 0 R\n",
+                       (long int)(objptr + 1), (long int)(objptr + 2),
+                       (long int)(objptr + 3), (long int)(objptr + 4));
         else 
-            PDF_PRINTF("/Length %li 0 R\n/Length1 %li 0 R\n" AND
-                       (long int)(objptr + 1) AND (long int)(objptr + 1));
-        PDF_PRINTF(">>\nstream\n");
+            pdf_printf("/Length %li 0 R\n/Length1 %li 0 R\n",
+                       (long int)(objptr + 1), (long int)(objptr + 1));
+        pdf_printf(">>\nstream\n");
     }
     for (k = 0; k < FONT_KEYS_NUM; k++)
         font_keys[k].valid = false;
@@ -173,82 +178,83 @@ void dopdffont(int objnum, internalfontnumber f)
         writet1();
     if (is_included() && !font_file_not_found) {
         if (!is_truetype()) {
-            PDF_PRINTF("endstream\nendobj\n");
+            pdf_printf("endstream\nendobj\n");
             pdfnewobj(0, 0);
-            PDF_PRINTF("%li\nendobj\n" AND
+            pdf_printf("%li\nendobj\n",
                        (long int)(t1_length1 + t1_length2 + t1_length3));
             pdfnewobj(0, 0);
-            PDF_PRINTF("%li\nendobj\n" AND (long int)t1_length1);
+            pdf_printf("%li\nendobj\n", (long int)t1_length1);
             pdfnewobj(0, 0);
-            PDF_PRINTF("%li\nendobj\n" AND (long int)t1_length2);
+            pdf_printf("%li\nendobj\n", (long int)t1_length2);
             pdfnewobj(0, 0);
-            PDF_PRINTF("%li\nendobj\n" AND (long int)t1_length3);
+            pdf_printf("%li\nendobj\n", (long int)t1_length3);
         }
         else {
-            PDF_PRINTF("endstream\nendobj\n");
+            pdf_printf("endstream\nendobj\n");
             pdfnewobj(0, 0);
-            PDF_PRINTF("%li\nendobj\n" AND (long int)ttf_length);
+            pdf_printf("%li\nendobj\n", (long int)ttf_length);
         }
     }
     if (!font_keys[FONTNAME_CODE].valid)
-        WARN("cannot read key `%s' from font file\nused value from map file" AND
+        pdftex_warn("cannot read key `%s' from font file, used value from map file",
              font_keys[FONTNAME_CODE].pdfname);
-    else if (fm_cur->base_name != 0 &&
+/* suppress basename mismatch; seems to be useless */
+/*
+    else if (fm_cur->base_name != 0 && fm_cur->extend == 0 && fm_cur->slant == 0 &&
              strcmp(font_keys[FONTNAME_CODE].value.s, fm_cur->base_name))
-        WARN("Base font name mismatch: `%s' (in font file) and  `%s' (in map file)!\nThe name specified in map file was ignored"
-             AND font_keys[FONTNAME_CODE].value.s 
-             AND fm_cur->base_name);
+        pdftex_warn("Base font name mismatch: `%s' (in font file) and  `%s' (in map file)!\nThe name specified in map file was ignored"
+, font_keys[FONTNAME_CODE].value.s 
+, fm_cur->base_name);
+*/
     pdfnewobj(0, 0); /* font base name */
-    PDF_PRINTF("/");
+    pdf_printf("/");
     if (fm_cur->prefix)
-        PDF_PRINTF("%s+" AND fm_cur->prefix);
+        pdf_printf("%s+", fm_cur->prefix);
     if (font_keys[FONTNAME_CODE].valid) {
-        PDF_PRINTF("%s" AND font_keys[FONTNAME_CODE].value.s);
+        pdf_printf("%s", font_keys[FONTNAME_CODE].value.s);
         XFREE(font_keys[FONTNAME_CODE].value.s);
     }
     else if (fm_cur->base_name != 0)
-        PDF_PRINTF("%s" AND fm_cur->base_name);
+        pdf_printf("%s", fm_cur->base_name);
     else
-        PDF_PRINTF("unknown!");
-    PDF_PRINTF("\nendobj\n");
+        pdf_printf("unknown!");
+    if (pdfexpandfont[f] != 0 && !is_included())
+        pdf_printf("%+li", pdfexpandfont[f]);
+    pdf_printf("\nendobj\n");
     pdfnewdict(0, 0); /* font descriptor */
     print_key(ASCENT_CODE, getcharheight(tex_font, 'h'));
     print_key(CAPHEIGHT_CODE, getcharheight(tex_font, 'H'));
     print_key(DESCENT_CODE, -getchardepth(tex_font, 'q'));
-    PDF_PRINTF("/FontName %li 0 R\n" AND (long int)(objptr - 1));
+    pdf_printf("/FontName %li 0 R\n", (long int)(objptr - 1));
     print_key(ITALIC_ANGLE_CODE, getitalicangle(tex_font));
     print_key(STEMV_CODE, getstemv(tex_font));
     print_key(XHEIGHT_CODE, getxheight(tex_font));
     if (!font_keys[FONTBBOX1_CODE].valid) {
-        /*
-        WARN("cannot read key `%s' from font file\nused value from %s.tfm"
-             AND font_keys[FONTBBOX1_CODE].pdfname
-             AND makecstring(fontname[tex_font]));
-        */
         getbbox(tex_font);
     }
-    PDF_PRINTF("/%s [ %li %li %li %li ]\n"
-               AND font_keys[FONTBBOX1_CODE].pdfname
-               AND (long int)font_keys[FONTBBOX1_CODE].value.i
-               AND (long int)font_keys[FONTBBOX2_CODE].value.i
-               AND (long int)font_keys[FONTBBOX3_CODE].value.i
-               AND (long int)font_keys[FONTBBOX4_CODE].value.i);
-    PDF_PRINTF("/Flags 00000%li\n" AND (long int)fm_cur->flags);
+    pdf_printf("/%s [ %li %li %li %li ]\n",
+               font_keys[FONTBBOX1_CODE].pdfname,
+               (long int)font_keys[FONTBBOX1_CODE].value.i,
+               (long int)font_keys[FONTBBOX2_CODE].value.i,
+               (long int)font_keys[FONTBBOX3_CODE].value.i,
+               (long int)font_keys[FONTBBOX4_CODE].value.i);
+    pdf_printf("/Flags %li\n", (long int)fm_cur->flags);
     if (is_subsetted() && cur_glyph_names != NULL) {
-        PDF_PRINTF("/CharSet (");
+        pdf_printf("/CharSet (");
         for (k = 0; k < MAX_CHAR_NUM; k++)
             if (pdfischarused(tex_font, k) && cur_glyph_names[k] != notdef)
-                PDF_PRINTF("/%s" AND cur_glyph_names[k]);
-        PDF_PRINTF(")\n");
+                pdf_printf("/%s", cur_glyph_names[k]);
+        pdf_printf(")\n");
         if (cur_glyph_names == builtin_glyph_names)
             for (k = 0; k < MAX_CHAR_NUM; k++)
                 if (cur_glyph_names[k] != notdef)
                     XFREE(cur_glyph_names[k]);
     }
-    if (is_included() && !font_file_not_found) 
+    if (is_included() && !font_file_not_found) {
         if (is_truetype())
-            PDF_PRINTF("/FontFile2 %li 0 R\n" AND (long int)(objptr - 3));
+            pdf_printf("/FontFile2 %li 0 R\n", (long int)(objptr - 3));
         else
-            PDF_PRINTF("/FontFile %li 0 R\n" AND (long int)(objptr - 6));
-    PDF_PRINTF(">> endobj\n");
+            pdf_printf("/FontFile %li 0 R\n", (long int)(objptr - 6));
+    }
+    pdf_printf(">> endobj\n");
 }
