@@ -50,7 +50,8 @@ typedef unsigned char byte;
 typedef struct {
     char *name;
     byte *data;
-    unsigned short len;
+    unsigned short len; /* length of the whole string */
+    unsigned short cslen; /* length of the encoded part of the string */
     boolean used;
 } cs_entry;
 
@@ -69,17 +70,15 @@ static FILE *t1_file;
 #define T1_EOF()        feof(t1_file)
 #define T1_PREFIX(s)    (!strncmp(t1_line, s, strlen(s)))
 #define T1_PUTCHAR(c)   pdfout(c)
-#define T1_CHARSTRINGS() (T1_PREFIX("2 index /CharStrings") || \
-                         T1_PREFIX("dup /CharStrings") ||  \
-                         T1_PREFIX("/CharStrings"))
+#define T1_CHARSTRINGS() strstr(t1_line, "/CharStrings")
 
 #define T1_CHECK_EOF()                                     \
     if (T1_EOF())                                          \
-        FAIL("unexpected end of file");
+        pdftex_fail("unexpected end of file");
 
 #define T1_PRINTF(S) do {                                  \
     sprintf(t1_line, S);                                   \
-    t1_line_ptr = strchr(t1_line, 0);                      \
+    t1_line_ptr = strend(t1_line);                      \
     t1_putline();                                          \
 } while (0)
 
@@ -100,10 +99,10 @@ static byte t1_getbyte()
         return c;
     if (t1_block_length == 0) {
         if (c != 128)
-            FAIL("invalid marker");
+            pdftex_fail("invalid marker");
         c = T1_GETCHAR();
         if (c == 3)
-            FAIL("unexpected end of file");
+            pdftex_fail("unexpected end of file");
         t1_block_length = T1_GETCHAR() & 0xff;
         t1_block_length |= (T1_GETCHAR() & 0xff) << 8;
         t1_block_length |= (T1_GETCHAR() & 0xff) << 16;
@@ -154,10 +153,20 @@ static byte eencrypt(byte plain)
     return cipher;
 }
 
+static char *eol(char *s)
+{
+    char *p = strend(s);
+    if (p - s > 1 && p[-1] != 10) {
+        *p++ = 10;
+        *p = 0;
+    }
+    return p;
+}
+
 static boolean t1_suffix(char *s) 
 {
     char *s1 = t1_line_ptr - 1, 
-         *s2 = strchr(s, 0) - 1;
+         *s2 = strend(s) - 1;
     if (*s1 == 10)
         s1--;
     while (s1 >= t1_line && s2 >= s) {
@@ -216,10 +225,87 @@ void t1_putline()
             T1_PUTCHAR(*p++);
 }
 
+void t1_modify_fm()
+{
+ /*
+  * font matrix is given as six numbers a0..a5, which stands for the matrix
+  * 
+  *           a0 a1 0
+  *     M =   a2 a3 0
+  *           a4 a5 1
+  * 
+  * ExtendFont is given as
+  * 
+  *           e 0 0
+  *     E =   0 1 0
+  *           0 0 1
+  * 
+  * SlantFont is given as
+  * 
+  *           1 0 0
+  *     S =   s 1 0
+  *           0 0 1
+  * 
+  * and the final transformation is
+  * 
+  *                    e*a0        e*a1       0
+  *     F =  E.S.M  =  s*e*a0+a2   s*e*a1+a3  0
+  *                    a4          a5         1
+  */
+    double e, s, a[6], b[6];
+    int i;
+    char buf[1024], *p, *q, *r;
+    if ((p = strchr(t1_line, '[')) == 0)
+        if ((p = strchr(t1_line, '{')) == 0)
+            pdftex_fail("an array expected: `%s'", t1_line);
+    strncpy(buf, t1_line, p - t1_line + 1);
+    for (i = 0, q = p + 1; i < 6; i++) {
+        a[i] = strtod(q, &r);
+        q = r + 1;
+    }
+    if (fm_cur->extend != 0)
+        e = fm_cur->extend*1E-3;
+    else
+        e = 1;
+    s = fm_cur->slant*1E-3;
+    b[0] = e*a[0];
+    b[1] = e*a[1];
+    b[2] = s*e*a[0] + a[2];
+    b[3] = s*e*a[1] + a[3];
+    b[4] = a[4];
+    b[5] = a[5];
+    q = buf + (p - t1_line + 1);
+    for (i = 0; i < 6; i++) {
+        sprintf(q, "%G ", b[i]);
+        q = strend(q);
+    }
+    strcpy(q, r);
+    strcpy(t1_line, buf);
+    t1_line_ptr = eol(t1_line);
+}
+
+void t1_modify_italic()
+{
+    double a;
+    char buf[1024], *p, *r;
+    if (fm_cur->slant == 0)
+        return;
+    p = strchr(t1_line, ' ');
+    strncpy(buf, t1_line, p - t1_line + 1);
+    a = strtod(p, &r);
+    a = a - atan(fm_cur->slant*1E-3)*(180/M_PI);
+    sprintf(buf + (p - t1_line + 1), "%.2g", a);
+    strcpy(strend(buf), r);
+    strcpy(t1_line, buf);
+    t1_line_ptr = eol(t1_line);
+    font_keys[ITALIC_ANGLE_CODE].value.i = (a > 0) ? a + 0.5 : a - 0.5;
+    font_keys[ITALIC_ANGLE_CODE].valid = true;
+}
+
 void t1_scan_param() 
 {
     int i, k;
-    char *p, buf[T1_BUF_SIZE], *q, *r;
+    char *p, buf[1024], *q, *r;
     key_entry *key;
     if (*t1_line != '/')
         return;
@@ -227,24 +313,46 @@ void t1_scan_param()
         t1_lenIV = atoi(strchr(t1_line, ' ') + 1);
         return;
     }
+    if (is_extended() || is_slanted()) {
+        if (strncmp(t1_line + 1, "UniqueID", strlen("UniqueID")) == 0) {
+            t1_line_ptr = t1_line;
+            return;
+        }
+        if (strncmp(t1_line + 1, "FontMatrix", strlen("FontMatrix")) == 0) {
+            t1_modify_fm();
+            return;
+        }
+        if (strncmp(t1_line + 1, "ItalicAngle", strlen("ItalicAngle")) == 0) {
+            t1_modify_italic();
+            return;
+        }
+    }
     for (key = font_keys; key - font_keys  < MAX_KEY_CODE; key++)
         if (!strncmp(t1_line + 1, key->t1name, strlen(key->t1name)))
           break;
     if (key - font_keys == MAX_KEY_CODE)
         return;
-    p = t1_line + strlen(key->t1name) + 1;
     key->valid = true;
+    p = t1_line + strlen(key->t1name) + 1;
     if (*p == ' ')
         p++;
     if ((k = key - font_keys) == FONTNAME_CODE) {
+        if (*p != '/')
+            pdftex_fail("a name expected: `%s'", t1_line);
         r = ++p; /* skip the slash */
         for (q = buf; *p != ' ' && *p != 10; *q++ = *p++);
         *q = 0;
+        if (is_extended()) {
+            sprintf(q, "-Extend_%i", (int)fm_cur->extend);
+        }
+        if (is_slanted()) {
+            sprintf(q, "-Slant_%i", (int)fm_cur->slant);
+        }
         key->value.s = xstrdup(buf);
         if (is_included() && is_subsetted()) {
             strcpy(buf, p);
             sprintf(r, "%s+%s%s", fm_cur->prefix, key->value.s, buf);
-            t1_line_ptr = strchr(r, 0);
+            t1_line_ptr = eol(r);
         }
         return;
     }
@@ -266,7 +374,7 @@ void t1_scan_param()
 void t1_builtin_enc()
 {
     int i, end_encoding = 0, counter = 0;
-    char buf[T1_BUF_SIZE], newline[T1_BUF_SIZE], *r, *q;
+    char buf[1024], line[1024], *r, *q;
        /*
         * At this moment "/Encoding" is the prefix of t1_line
         * 
@@ -284,8 +392,8 @@ void t1_builtin_enc()
         */
     if (T1_PREFIX("/Encoding [") || T1_PREFIX("/Encoding[")) { /* the first case */
         r = strchr(t1_line, '[') + 1;
-        strncpy(newline, t1_line, r - t1_line);
-        newline[r - t1_line + 1] = 0;
+        strncpy(line, t1_line, r - t1_line);
+        line[r - t1_line + 1] = 0;
         if (*r == 32)
             r++;
         for(;;) {
@@ -296,67 +404,60 @@ void t1_builtin_enc()
                     r++;
                 if (strcmp(buf, notdef) != 0 && pdfischarused(tex_font, counter))
                     builtin_glyph_names[counter] = xstrdup(buf);
-                sprintf(strchr(newline, 0), "/%s", buf);
+                sprintf(strend(line), "/%s", buf);
                 counter++;
             }
-            if (*r != 10 && *r != '%')
+            if (*r != 10 && *r != '%') {
                 if (strncmp(r, "] def", strlen("] def")) == 0 ||
                     strncmp(r, "] readonly def", strlen("] readonly def")) == 0) {
                     end_encoding = 1;
-                    strcat(newline, r);
+                    strcat(line, r);
                 }
                 else
-                    FAIL("a name or `] def' or `] readonly def' expected:\n%s" AND t1_line);
-            strcpy(t1_line, newline);
-            t1_line_ptr = strchr(t1_line, 0);
-            APPEND_EOL(t1_line_ptr, t1_line, T1_BUF_SIZE);
+                    pdftex_fail("a name or `] def' or `] readonly def' expected: `%s'", t1_line);
+            }
+            strcpy(t1_line, line);
+            t1_line_ptr = eol(t1_line);
             t1_putline();
             if (end_encoding)
                 break;
             T1_CHECK_EOF();
             t1_getline();
             r = t1_line;
-            *newline = 0;
+            *line = 0;
         }
     }
     else { /* the second case */
-        do {
-            t1_putline();
-            T1_CHECK_EOF();
-            t1_getline();
-            strncpy(newline, t1_line, strlen(t1_line) - 1);
-            /* truncate the end-of-line character */
-        } while
-        (strstr("/Encoding 256 array 0 1 255 {1 index exch /.notdef put} for", newline) != NULL ||
-         strstr("/Encoding 256 array 0 1 255{1 index exch/.notdef put}for", newline) != NULL);
-        do {
-            if (sscanf(t1_line, "dup %u%s put", &i, buf) == 2 &&
-                *buf == '/' && pdfischarused(tex_font, i)) {
-                builtin_glyph_names[i] = xstrdup(buf + 1); /* skip the slash */
-                t1_putline();
-            }
-            T1_CHECK_EOF();
-            t1_getline();
-        } while (!t1_suffix("def"));
         t1_putline();
+        do {
+            T1_CHECK_EOF();
+            t1_getline();
+            if (sscanf(t1_line, "dup %u%256s put", &i, buf) == 2 && *buf == '/') {
+                if (pdfischarused(tex_font, i))
+                    builtin_glyph_names[i] = xstrdup(buf + 1); /* skip the slash */
+                else
+                    continue;
+            }
+            t1_putline();
+        } while (!t1_suffix("def"));
     }
 }
 
 void cs_store()
 {
     char *p, *q, buf[T1_BUF_SIZE];
-    for (p = t1_line, q = buf; *p != ' ';)
-        *q++ = *p++;
+    for (p = t1_line, q = buf; *p != ' '; *q++ = *p++);
     *q = 0;
     cs_ptr->name = xstrdup(buf + 1); /* don't store the slash */
-    cs_ptr->len = t1_cslen;
-    cs_ptr->data = XTALLOC(cs_ptr->len + 8, byte);
     cs_ptr->used = false;
-    memcpy(cs_ptr->data, cs_start - 4, cs_ptr->len + 6);
-    p = cs_start - 4 + cs_ptr->len + 6;
-    q = cs_ptr->data + cs_ptr->len + 6;
-    for (; *p != 10; *q++ = *p++);
-    *q = 10;
+    memcpy(buf, cs_start - 4, t1_cslen + 4); /* copy " RD " + cs data to buf */
+    for (p = cs_start + t1_cslen, q = buf + t1_cslen + 4; *p != 10; *q++ = *p++);
+    /* copy the end of cs data to buf */
+    *q++ = 10; 
+    cs_ptr->len = q - buf;
+    cs_ptr->cslen = t1_cslen;
+    cs_ptr->data = XTALLOC(cs_ptr->len, byte);
+    memcpy(cs_ptr->data, buf, cs_ptr->len);
     cs_ptr++;
 }
 
@@ -376,7 +477,7 @@ static void cs_mark(char *name)
             else {
                 cs->used = true;
                 cr = 4330; 
-                cs_len = cs->len;
+                cs_len = cs->cslen;
                 data = cs->data + 4;
                 for (i = 0; i < t1_lenIV; i++, cs_len--)
                     cs_getchar();
@@ -418,22 +519,34 @@ static void cs_mark(char *name)
             }
         }
     }
-    WARN("glyph `%s' undefined" AND name);
+    pdftex_warn("glyph `%s' undefined", name);
 }
 
 void writet1()
 {
     int i, size_pos, cs_dict_size, cs_count, encoding;
-    char buf[T1_BUF_SIZE], *save_line1, *save_line2, *p, *q;
+    char buf[T1_BUF_SIZE], *save_line1, *save_line2, *p;
     cs_entry *cs;
+    if (fm_cur->mm_name != 0) {
+        filename = fm_cur->mm_name;
+        packfilename(maketexstring(filename), getnullstr(), getnullstr());
+        if (T1_OPEN()) /* found mm instance */
+            goto open_ok;
+    }
     filename = fm_cur->ff_name;
     packfilename(maketexstring(filename), getnullstr(), getnullstr());
     if (!T1_OPEN()) {
-        WARN("cannot open Type 1 font file for reading");
+        pdftex_warn("cannot open Type 1 font file for reading");
         font_file_not_found = true;
         return;
     }
-    TEX_PRINTF("<%s" AND fm_cur->ff_name);
+    if (pdfexpandfont[f] != 0) { /* cannot open mm instance, use ExtendFont */
+        if (fm_cur->extend == 0)
+            fm_cur->extend = 1000;
+        fm_cur->extend = xnoverd(fm_cur->extend, 1000 + pdfexpandfont[f], 1000);
+    }
+open_ok:
+    tex_printf("<%s", fm_cur->ff_name);
     t1_lenIV = 4;
     t1_dr = 55665;
     t1_er = 55665;
@@ -458,7 +571,7 @@ void writet1()
             t1_getline();
             t1_scan_param();
         } while (!(T1_CHARSTRINGS() || T1_PREFIX("/Subrs")));
-        TEX_PRINTF(">");
+        tex_printf(">");
         T1_CLOSE();
         return;
     }
@@ -501,7 +614,7 @@ void writet1()
             t1_putline();
         } while (!T1_PREFIX("cleartomark"));
         t1_length3 = pdfoffset() - pdfsaveoffset;
-        TEX_PRINTF(">");
+        tex_printf(">");
         T1_CLOSE();
         return;
     } 
@@ -516,12 +629,12 @@ void writet1()
         t1_getline();
     }
     if (t1_suffix("def")) {
-        sscanf(t1_line + strlen("/Encoding"), "%s", buf);
+        sscanf(t1_line + strlen("/Encoding"), "%256s", buf);
         if (!strcmp(buf, "StandardEncoding"))
             encoding = ENC_STANDARD;
         else 
-            FAIL("cannot subset font (unknown predefined encoding `%s')"
-                 AND buf);
+            pdftex_fail("cannot subset font (unknown predefined encoding `%s')"
+, buf);
     }
     else {
         encoding = ENC_BUITIN; /* the font has its own built-in encoding */
@@ -590,12 +703,12 @@ void writet1()
     else if (encoding == ENC_STANDARD)
         cur_glyph_names = standard_glyph_names;
     for (i = 0; i < MAX_CHAR_NUM; i++)
-        if (pdfischarused(tex_font, i))
+        if (pdfischarused(tex_font, i)) {
             if (cur_glyph_names[i] == notdef)
-                WARN("character %i is mapped to %s" AND
-                     i AND notdef);
+                pdftex_warn("character %i is mapped to %s", i, notdef);
             else
                 cs_mark(cur_glyph_names[i]);
+        }
     cs_mark(notdef);
     for (cs_count = 0, cs = cs_tab; cs < cs_ptr; cs++)
         if (cs->used)
@@ -607,24 +720,20 @@ void writet1()
     sprintf(t1_line_ptr, "%u", cs_count);
     strcat(t1_line_ptr, p);
     XFREE(save_line1);
-    t1_line_ptr = strchr(t1_line_ptr, 0);
+    t1_line_ptr = eol(t1_line);
     t1_putline();
     for (cs = cs_tab; cs < cs_ptr; cs++) {
         if (cs->used) {
-            sprintf(t1_line, "/%s %u", cs->name, cs->len);
-            p = strchr(t1_line, 0);
-            memcpy(p, cs->data, cs->len + 6);
-            p += cs->len + 6;
-            q = cs->data + cs->len + 6;
-            for (; *q != 10; *p++ = *q++);
-            *p++ = 10;
-            t1_line_ptr = p;
+            sprintf(t1_line, "/%s %u", cs->name, cs->cslen);
+            p = strend(t1_line);
+            memcpy(p, cs->data, cs->len);
+            t1_line_ptr = p + cs->len;
             t1_putline();
         }
         XFREE(cs->data);
     }
     sprintf(t1_line, "%s", save_line2);
-    t1_line_ptr = strchr(t1_line, 0);
+    t1_line_ptr = eol(t1_line);
     t1_putline();
     XFREE(save_line2);
     while (!t1_suffix("mark currentfile closefile")) {
@@ -642,7 +751,7 @@ void writet1()
         t1_putline();
     }
     t1_length3 = pdfoffset() - pdfsaveoffset;
-    TEX_PRINTF(">");
+    tex_printf(">");
     XFREE(cs_tab);
     T1_CLOSE();
 }
