@@ -1,6 +1,6 @@
 /*========================================================================*\
 
-Copyright (c) 1990-1997  Paul Vojta
+Copyright (c) 1990-1999  Paul Vojta
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -38,6 +38,10 @@ NOTE:
 
 #include "c-auto.h"
 
+#ifdef __linux
+#define FLAKY_SIGPOLL 1
+#endif
+
 #if STDC_HEADERS
 # include <stddef.h>
 # include <stdlib.h>
@@ -143,6 +147,7 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 # endif
 #endif
 
+/* These are for X headers */
 #ifndef	NeedFunctionPrototypes
 #if	__STDC__
 #define	NeedFunctionPrototypes	1
@@ -151,12 +156,46 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 #endif	/* STDC */
 #endif	/* NeedFunctionPrototypes */
 
-#if	NeedFunctionPrototypes
+#ifndef HAVE_PROTOTYPES
+#if	__STDC__
+#define HAVE_PROTOTYPES 1
+#else   /* STDC */
+#define HAVE_PROTOTYPES 0
+#endif  /* STDC */
+#endif
+
+/* These are for xdvi */
+#if	HAVE_PROTOTYPES
 #define	ARGS(x)	x
 #else
 #define	ARGS(x)	()
 #endif
 
+#ifndef KPATHSEA
+
+/* These macros munge function declarations to make them work in both
+   cases.  The P?H macros are used for declarations, the P?C for
+   definitions.  See <ansidecl.h> from the GNU C library.  P1H(void)
+   also works for definitions of routines which take no args.  */
+
+#if __STDC__
+
+#define	P1H(p1) (p1)
+
+#define	P1C(t1,n1)(t1 n1)
+#define	P2C(t1,n1, t2,n2)(t1 n1, t2 n2)
+
+#else /* not __STDC__ */
+
+#define	P1H(p1) ()
+
+#define	P1C(t1,n1) (n1) t1 n1;
+#define	P2C(t1,n1, t2,n2) (n1,n2) t1 n1; t2 n2;
+
+#endif /* not __STDC__ */
+
+#endif /* not KPATHSEA */
+
 #ifndef	NeedWidePrototypes
 #define	NeedWidePrototypes	NeedFunctionPrototypes
 #endif
@@ -216,6 +255,11 @@ typedef	char		Bool3;		/* Yes/No/Maybe */
 #define	Strcat	(void) strcat
 #define	Strcpy	(void) strcpy
 
+#ifdef HTEX
+char *urlocalize P1H(char *filename);
+extern int lastwwwopen;
+#endif
+
 /********************************
  *	 Types and data		*
  *******************************/
@@ -248,7 +292,7 @@ typedef	Boolean		wide_bool;
  *	to pixel values; although normally it should be
  *		((int) ((x) / shrink_factor + (1 << 15) >> 16)),
  *	the rounding is achieved instead by moving the constant 1 << 15 to
- *	PAGE_OFFSET in dvi_draw.c.
+ *	PAGE_OFFSET in dvi-draw.c.
  */
 #define	pixel_conv(x)		((int) ((x) / shrink_factor >> 16))
 #define	pixel_round(x)		((int) ROUNDUP(x, shrink_factor << 16))
@@ -679,6 +723,7 @@ EXTERN	Pixel		*pixeltbl_t;
 EXTERN	Boolean		canit		INIT(False);
 EXTERN	jmp_buf		canit_env;
 EXTERN	VOLATILE short	event_counter	INIT(0);
+EXTERN	Boolean		terminate_flag	INIT(False);
 
 struct	WindowRec {
 	Window		win;
@@ -741,17 +786,17 @@ struct cfg2res {
 #if	PS
 	
 extern	struct psprocs	{
-	void	(*toggle) ARGS((void));
-	void	(*destroy) ARGS((void));
-	void	(*interrupt) ARGS((void));
-	void	(*endpage) ARGS((void));
-	void	(*drawbegin) ARGS((int, int, _Xconst char *));
-	void	(*drawraw) ARGS((_Xconst char *));
-	void	(*drawfile) ARGS((_Xconst char *, FILE *));
-	void	(*drawend) ARGS((_Xconst char *));
-	void	(*beginheader) ARGS((void));
-	void	(*endheader) ARGS((void));
-	void	(*newdoc) ARGS((void));
+	void	(*toggle)	ARGS((void));
+	void	(*destroy)	ARGS((void));
+	void	(*interrupt)	ARGS((void));
+	void	(*endpage)	ARGS((void));
+	void	(*drawbegin)	ARGS((int, int, _Xconst char *));
+	void	(*drawraw)	ARGS((_Xconst char *));
+	void	(*drawfile)	ARGS((_Xconst char *, FILE *));
+	void	(*drawend)	ARGS((_Xconst char *));
+	void	(*beginheader)	ARGS((void));
+	void	(*endheader)	ARGS((void));
+	void	(*newdoc)	ARGS((void));
 }	psp, no_ps_procs;
 
 #endif	/* PS */
@@ -834,21 +879,22 @@ extern	void	applicationDoSpecial ARGS((char *));
 #if	PS
 extern	void	scan_special ARGS((char *));
 #endif
-extern	NORETURN void	oops ARGS((_Xconst char *, ...));
+extern	NORETURN void	oops ARGS((_Xconst char *message, ...));
 #ifndef KPATHSEA
 extern	void	*xmalloc ARGS((unsigned));
 extern	void	*xrealloc ARGS((void *, unsigned));
+extern	char	*xstrdup ARGS((_Xconst char *, int));
+extern	char	*xmemdump ARGS((_Xconst char *, int));
 #endif
-extern	char	*newstring ARGS((_Xconst char *, int));
 extern	void	expandline ARGS((int));
 extern	void	alloc_bitmap ARGS((struct bitmap *));
-#ifdef KPATHSEA
+#ifndef KPATHSEA
 extern	void	xputenv ARGS((_Xconst char *, _Xconst char *));
 #endif
 extern	int	memicmp ARGS((_Xconst char *, _Xconst char *, size_t));
-extern	FILE	*xfopen ARGS((_Xconst char *, OPEN_MODE_ARGS));
+extern FILE    *xfopen P2H(char *, OPEN_MODE_ARGS);
 #ifdef HTEX
-extern	FILE	*xfopen_local ARGS((_Xconst char *, OPEN_MODE_ARGS));
+extern FILE    *xfopen_local P2H(char *, OPEN_MODE_ARGS);
 #else
 #define xfopen_local	xfopen
 #endif
@@ -881,11 +927,9 @@ extern	Boolean	initGS ARGS((void));
 #endif	/* PS */
 #ifdef HTEX
 extern	int	open_www_file ARGS((void));
-extern	void	cleanup_and_exit ARGS((int));
 extern	void	htex_can_it ARGS((void));
 extern	void	search_callback ARGS((Widget, XtPointer, XtPointer));
 extern	void	detach_anchor ARGS((void));
-extern	FILE	*xfopen_local ARGS((_Xconst char *, OPEN_MODE_ARGS));
 extern	char	*MyStrAllocCopy ARGS((char **, char *));
 extern	int	pointerlocate ARGS((int *, int *));
 extern	void	htex_recordbits ARGS((int, int, int, int));
@@ -914,6 +958,11 @@ extern	void	htex_do_url ARGS((char *));
 extern  void 	paint_anchor ARGS((char *));
 extern	int	fetch_relative_url ARGS((char *, const char *, char *));
 extern	void	wait_for_urls ARGS((void));
+char *figure_mime_type ARGS((char *));
+#endif
+
+#if !HAVE_TEMPNAM
+char *tempnam ARGS((char *, char *));
 #endif
 
 _XFUNCPROTOEND
@@ -937,7 +986,6 @@ typedef struct {
 EXTERN FiletoURLconv *filelist INIT(NULL);
 EXTERN int nURLs INIT(0);
 
-#define exit(n) cleanup_and_exit(n)
 #endif /* HTEX */
 
 #endif	/* XDVI_H */
